#ifndef BOARD_H
#define BOARD_H

#include <vector>
#include <stdexcept>
#include <iostream>

/// @brief A template class representing a board
/// @tparam T The cell type
template <typename T>
class Board
{
    /// @brief The width of the board
    uint32_t m_width;
    /// @brief The height of the board
    uint32_t m_height;
    /// @brief The board data
    std::vector<T> m_data;
public:
    /// @brief Create a board where all elements are the same
    /// @param width   The width of the board
    /// @param height  The height of the board
    /// @param element The element in every cell
    Board(uint32_t width, uint32_t height, const T& element) : m_width(width), m_height(height)
    { m_data.resize(m_width * m_height, element); };
    
    /// @brief Read a board from a file
    /// @param file A file pointer
    Board(std::FILE* file);

    /// @brief Getter for the board width 
    uint32_t width() const { return m_width; }
    
    /// @brief Getter for the board height 
    uint32_t height() const { return m_height; }

    /// @brief Getter for a single board element
    /// @param x The elements x coordinate
    /// @param y The elements y coordinate
    /// @return T& A reference to the element
    T& at(uint32_t x, uint32_t y) { return m_data[x + m_width * y]; }
    
    /// @brief Method that prints the entire board
    void print();
};

/// @brief Special exception for Board errors
struct BoardException : public std::logic_error {
    BoardException(const char* string) : std::logic_error(string) {};
};

template <typename T>
void Board<T>::print() {
    for (uint32_t h = 0; h < m_height; h++)
    {
        for(uint32_t w = 0; w < m_width; w++)
        {
            std::cout << to_string(at(w, h));
        }
        std::cout << std::endl;
    }
}

#endif // BOARD_H
