#include <z3++.h>
#include <cstdio>
#include "Board.h"

int main()
{
    z3::context ctx;
    z3::solver solver(ctx);

    std::array<std::array<int, 9>, 9> problem = {{
        {0, 6, 0, 1, 0, 0, 4, 0, 0},
        {0, 0, 5, 0, 0, 0, 0, 0, 0},
        {9, 2, 0, 3, 4, 7, 5, 6, 0},
        {6, 0, 2, 0, 1, 3, 0, 8, 0},
        {8, 0, 1, 0, 9, 0, 0, 5, 7},
        {0, 0, 0, 0, 0, 2, 0, 1, 0},
        {1, 5, 0, 4, 0, 9, 0, 3, 2},
        {3, 0, 0, 0, 8, 1, 7, 4, 0},
        {0, 0, 4, 7, 0, 5, 0, 9, 0}
    }};

    Board<z3::expr> vars(9, 9, z3::expr(ctx));

    for (int i = 0; i < 9; i++)
    {
        for (int j = 0; j < 9; j++)
        {
            if (problem[i][j] == 0)
            {
                vars.at(i, j) = ctx.int_const(("x_" + std::to_string(i) + "_" + std::to_string(j)).c_str());
                solver.add(vars.at(i,j) >= 1);
                solver.add(vars.at(i,j) <= 9);
            }
            else
                vars.at(i, j) = ctx.int_val(problem[i][j]);
        }
    }
    
    for (int i = 0; i < 9; i++)
    {
        // z3::distinct
        z3::expr_vector row(ctx);
        for (int j = 0; j < 9; j++)
            row.push_back(vars.at(i,j));
        solver.add(z3::distinct(row));
    }

    for (int j = 0; j < 9; j++)
    {
        // z3::distinct
        z3::expr_vector col(ctx);
        for (int i = 0; i < 9; i++)
            col.push_back(vars.at(i,j));
        solver.add(z3::distinct(col));
    }

    for (int i = 0; i < 9; i += 3)
    for (int j = 0; j < 9; j += 3)
    {
        z3::expr_vector box(ctx);
        for (int ii = 0; ii < 3; ii++)
        for (int jj = 0; jj < 3; jj++)
            box.push_back(vars.at(i+ii,j+jj));
        solver.add(z3::distinct(box));
    }


    if (solver.check() == z3::sat)
    {
        z3::model model = solver.get_model();
        for (int i = 0; i < 9; i++)
        {
            for (int j = 0; j < 9; j++)
            {
                printf("%ld ", model.eval(vars.at(i,j)).as_int64());
            }
            printf("\n");
        }
    }

    return 0;
}
